package cmd

import (
	"os"
	"fmt"
	"strings"

	"github.com/spf13/cobra"
	"github.com/spf13/viper"
)

const (
	version = "0.02"
)

var (
	gJails			[]Jail

	gUseSudo		bool

	gConfigFile		string
	gDisplayColumns	string
	gFilterJails	string
	gSortFields		string
	gNoLineSep		bool


	rootCmd = & cobra.Command{
    	Use:    "gocage",
	    Short:  "GoCage is a FreeBSD Jail management tool",
	    Long:   `GoCage is a jail management tool. It support VNET, host-only, NAT networks. Provides snapshots and cloning.
It support iocage jails and can coexist with iocage.`,

	    Run: func(cmd *cobra.Command, args []string) {
	        fmt.Println("Here we are in the Run")
	    },
	}

	versionCmd = &cobra.Command{
	    Use:   "version",
	    Short: "Print the version number of GoCage",
	    Long:  `Let this show you how much fail I had to get this *cough* perfect`,
	    Run: func(cmd *cobra.Command, args []string) {
	        fmt.Printf("GoCage v.%s\n", version)
	    },
	}

	listCmd = &cobra.Command{
		Use:   "list",
		Short: "Print jails",
		Long:  `Display jails, their IP and OS.
Jail list can be restricted by adding name on command line
ex: gocage list srv-db srv-web`,
		Run: func(cmd *cobra.Command, args []string) {
			ListJails(args, true)
		},
	}
	
	stopCmd = &cobra.Command{
		Use:   "stop",
		Short: "stop jail",
		Long:  `shutdown jail`,
		Run: func(cmd *cobra.Command, args []string) {
			// Get the inventory
			ListJails(args, false)
			StopJail(args)
		},
	}
)



func init() {
    cobra.OnInitialize(initConfig)

// Global switches
    rootCmd.PersistentFlags().StringVarP(&gConfigFile, "config", "c", "/usr/local/etc/gocage.conf.yml", "GoCage configuration file")
    rootCmd.PersistentFlags().BoolVarP(&gUseSudo, "sudo", "u", false, "Use sudo to run commands")

// Command dependant switches
    listCmd.PersistentFlags().StringVarP(&gDisplayColumns, "outcol", "o", "JID,Name,Config.Release,Config.Ip4_addr,Running", "Show these columns in output")
    listCmd.PersistentFlags().BoolVarP(&gNoLineSep, "nolinesep", "l", false, "Do not display line separator between jails")
    listCmd.PersistentFlags().StringVarP(&gFilterJails, "filter", "f", "none", "Only display jails with these values. Ex: \"gocage list -f Config.Boot=1\" will only list started on boot jails")
    listCmd.PersistentFlags().StringVarP(&gSortFields, "sort", "s", "none", "Display jails sorted by field values. Ex: \"gocage list -s +Name,-Config.Priority\" will sort jails by their decreasing name, then increasing start priority. 3 critera max supported.")
    
// Now declare commands
    rootCmd.AddCommand(versionCmd)
    rootCmd.AddCommand(listCmd)
    rootCmd.AddCommand(stopCmd)
}

func initConfig() {
	if gConfigFile == "" {
		fmt.Println("No config file!")
		os.Exit(1)
	}

//	fmt.Printf("We are in initConfig(), with config file %s\n", gConfigFile)

	viper.SetConfigFile(gConfigFile)

	if err := viper.ReadInConfig(); err != nil {
		fmt.Printf("ERROR reading config file %s : %s\n", gConfigFile, err.Error())
		os.Exit(1)
	}

//	fmt.Println("Using config file:", viper.ConfigFileUsed())
//	fmt.Printf("datastore in config : %s\n", viper.GetStringSlice("datastore"))
//	fmt.Printf("datastore.0 in config : %s\n", viper.GetStringSlice("datastore.0"))
	
	// Command line flags have priority on config file
	if rootCmd.Flags().Lookup("sudo") != nil && false == rootCmd.Flags().Lookup("sudo").Changed {
		gUseSudo = viper.GetBool("sudo")
	}
	if listCmd.Flags().Lookup("outcol") != nil && false == listCmd.Flags().Lookup("outcol").Changed {
		gDisplayColumns = viper.GetString("outcol")
	}
	if listCmd.Flags().Lookup("nolinesep") != nil && false == listCmd.Flags().Lookup("nolinesep").Changed {
		gNoLineSep = viper.GetBool("nolinesep")
	}
	if listCmd.Flags().Lookup("filter") != nil && false == listCmd.Flags().Lookup("filter").Changed {
		gFilterJails = viper.GetString("filter")
	}
	if listCmd.Flags().Lookup("sort") != nil && false == listCmd.Flags().Lookup("sort").Changed {
		gSortFields = viper.GetString("sort")
	}
	if len(strings.Split(gSortFields, ",")) > 3 {
		fmt.Printf("More than 3 sort criteria, this is not supported!\n")
		os.Exit(1)
	}
}

func Execute() {
    if err := rootCmd.Execute(); err != nil {
        fmt.Fprintln(os.Stderr, err)
        os.Exit(1)
    }
}


